<?php

/*
  $Id: $

  osCommerce, Open Source E-Commerce Solutions
  http://www.oscommerce.com

  Copyright (c) 2020 osCommerce

  Released under the GNU General Public License
 */

require_once DIR_FS_CATALOG . 'includes/apps/stripe_sca/init.php';

class stripe_sca extends abstract_payment_module {

    const CONFIG_KEY_BASE = 'MODULE_PAYMENT_STRIPE_SCA_';
    const REQUIRES = [ 'name', 'street_address', 'postcode', 'city', 'country', 'zone_id', 'email_address', 'id' ];

    public $intent;
    private $signature = 'stripe|stripe_sca|1.1.0|2.3';
    public $api_version = '2019-08-14';

    function __construct() {
        global $order, $payment;

        parent::__construct();
        $this->order_status = defined('MODULE_PAYMENT_STRIPE_SCA_PREPARE_ORDER_STATUS_ID') && ((int) MODULE_PAYMENT_STRIPE_SCA_PREPARE_ORDER_STATUS_ID > 0) ? (int) MODULE_PAYMENT_STRIPE_SCA_PREPARE_ORDER_STATUS_ID : 0;

        if (defined('MODULE_PAYMENT_STRIPE_SCA_STATUS')) {
            if (MODULE_PAYMENT_STRIPE_SCA_TRANSACTION_SERVER == 'Test') {
                $this->title .= ' [Test]';
                $this->public_title .= ' (Test)';
            }

            $this->description .= $this->getTestLinkInfo();
        }

        if (!function_exists('curl_init')) {
            $this->description = '<div class="secWarning">' . MODULE_PAYMENT_STRIPE_SCA_ERROR_ADMIN_CURL . '</div>' . $this->description;

            $this->enabled = false;
        }

        if ($this->enabled === true) {
            if ((MODULE_PAYMENT_STRIPE_SCA_TRANSACTION_SERVER == 'Live' && (!tep_not_null(MODULE_PAYMENT_STRIPE_SCA_LIVE_PUBLISHABLE_KEY) || !tep_not_null(MODULE_PAYMENT_STRIPE_SCA_LIVE_SECRET_KEY))) || (MODULE_PAYMENT_STRIPE_SCA_TRANSACTION_SERVER == 'Test' && (!tep_not_null(MODULE_PAYMENT_STRIPE_SCA_TEST_PUBLISHABLE_KEY) || !tep_not_null(MODULE_PAYMENT_STRIPE_SCA_TEST_SECRET_KEY)))) {
                $this->description = '<div class="secWarning">' . MODULE_PAYMENT_STRIPE_SCA_ERROR_ADMIN_CONFIGURATION . '</div>' . $this->description;

                $this->enabled = false;
            } elseif (isset($order) && $order instanceof order) {
                $this->update_status();
            }
        }

        if (('modules.php' === $GLOBALS['PHP_SELF']) && ('install' === ($_GET['action'] ?? null)) && ('conntest' === ($_GET['subaction'] ?? null))) {
            echo $this->getTestConnectionResult();
            exit;
        }
    }

    private function extract_order_id() {
      return substr($_SESSION['cart_Stripe_SCA_ID'], strpos($_SESSION['cart_Stripe_SCA_ID'], '-')+1);
    }

    function selection() {
        if ((MODULE_PAYMENT_STRIPE_SCA_TOKENS == 'True') && !isset($_SESSION['payment'])) {
            $tokens_query = tep_db_query("SELECT 1 FROM customers_stripe_tokens WHERE customers_id = " . (int) $_SESSION['customer_id'] . " LIMIT 1");

            if (tep_db_num_rows($tokens_query)) {
              $_SESSION['payment'] = $this->code;
            }
        }

        return [
          'id' => $this->code,
          'module' => $this->public_title,
        ];
    }

    function pre_confirmation_check() {
        global $oscTemplate;

        if ($this->templateClassExists()) {
            $oscTemplate->addBlock($this->getSubmitCardDetailsJavascript(), 'footer_scripts');
        }
    }

    function confirmation() {
        global $languages_id, $order, $currency, $shipping;

        if (isset($_SESSION['cartID'])) {
            $insert_order = false;

            if (isset($_SESSION['cart_Stripe_SCA_ID'])) {
                $order_id = $this->extract_order_id();

                $curr_check = tep_db_query("select currency from orders where orders_id = " . (int) $order_id);
                $curr = tep_db_fetch_array($curr_check);

                if (($curr['currency'] != $order->info['currency']) || ($_SESSION['cartID'] != substr($_SESSION['cart_Stripe_SCA_ID'], 0, strpos($_SESSION['cart_Stripe_SCA_ID'], '-')))) {
                    $check_query = tep_db_query('select orders_id from orders_status_history where orders_id = ' . (int) $order_id . ' limit 1');

                    if (tep_db_num_rows($check_query) < 2) {
                      tep_delete_order($order_id);
                    }

                    $insert_order = true;
                }
            } else {
                $insert_order = true;
            }

            if ($insert_order) {

                if (isset($order->info['payment_method_raw'])) {
                    $order->info['payment_method'] = $order->info['payment_method_raw'];
                    unset($order->info['payment_method_raw']);
                }
                
                $GLOBALS['customer_notification'] = 0;

                require 'includes/system/segments/checkout/build_order_totals.php';
                require 'includes/system/segments/checkout/insert_order.php';
                require 'includes/system/segments/checkout/insert_history.php';

                $_SESSION['cart_Stripe_SCA_ID'] = $_SESSION['cartID'] . '-' . $order_id;
            }
        }

        $secret_key = MODULE_PAYMENT_STRIPE_SCA_TRANSACTION_SERVER == 'Live' ? MODULE_PAYMENT_STRIPE_SCA_LIVE_SECRET_KEY : MODULE_PAYMENT_STRIPE_SCA_TEST_SECRET_KEY;
        \Stripe\Stripe::setApiKey($secret_key);
        \Stripe\Stripe::setApiVersion($this->api_version);

        $metadata = [
          'customer_id' => tep_output_string($_SESSION['customer_id']),
          'order_id' => tep_output_string($order_id),
          'company' => isset($order->customer['company'])? tep_output_string($order->customer['company']) : '',
        ];

        $content = '';

        if (MODULE_PAYMENT_STRIPE_SCA_TOKENS == 'True') {
            $tokens_query = tep_db_query("SELECT id, stripe_token, card_type, number_filtered, expiry_date FROM customers_stripe_tokens WHERE customers_id = " . (int) $_SESSION['customer_id'] . " ORDER BY date_added");

            if (tep_db_num_rows($tokens_query) > 0) {
                $content .= '<table id="stripe_table" border="0" width="100%" cellspacing="0" cellpadding="2">';

                while ($tokens = tep_db_fetch_array($tokens_query)) {
                    // default to charging first saved card, changed by client directly calling payment_intent.php hook as selection changed
                    $content .= '<tr class="moduleRow" id="stripe_card_' . (int) $tokens['id'] . '">' .
                            '  <td width="40" valign="top"><input type="radio" name="stripe_card" value="' . (int) $tokens['id'] . '" /></td>' .
                            '  <td valign="top"><strong>' . tep_output_string_protected($tokens['card_type']) . '</strong>&nbsp;&nbsp;****' . tep_output_string_protected($tokens['number_filtered']) . '&nbsp;&nbsp;' . tep_output_string_protected(substr($tokens['expiry_date'], 0, 2) . '/' . substr($tokens['expiry_date'], 2)) . '</td>' .
                            '</tr>';
                }

                $content .= '<tr class="moduleRow" id="stripe_card_0">' .
                        '  <td width="40" valign="top"><input type="radio" name="stripe_card" value="0" /></td>' .
                        '  <td valign="top">' . MODULE_PAYMENT_STRIPE_SCA_CREDITCARD_NEW . '</td>' .
                        '</tr>' .
                        '</table><div id="save-card-element"></div>';
            }
        }

        $content .= '<div id="stripe_table_new_card">' .
                '<div class="form-group"><label for="cardholder-name" class="control-label">' . MODULE_PAYMENT_STRIPE_SCA_CREDITCARD_OWNER . '</label>' .
                '<div class="col-sm-16"><input type="text" id="cardholder-name" class="form-control" value="' . tep_output_string($order->billing['name']) . '" required></div></div>' .
                '<div class="form-group"><label for="card-element" class="control-label">' . MODULE_PAYMENT_STRIPE_SCA_CREDITCARD_TYPE . '</label>' .
                '<div id="card-element" class="col-sm-16"></div></div>';
        if (MODULE_PAYMENT_STRIPE_SCA_TOKENS == 'True') {
          $content .= '<div class="form-group row align-items-center">
                       <div class="col-sm-9 pl-5 ml-2 custom-control custom-switch">' . tep_draw_checkbox_field('card-save', 1, false, 'class="custom-control-input" id="inputCardSave"') . 
                      ' <label for="inputCardSave" class="custom-control-label text-muted">' . MODULE_PAYMENT_STRIPE_SCA_CREDITCARD_SAVE . '</label>
                        </div>
                      </div>';
        }
        $content .= '</div><div id="card-errors" role="alert" class="messageStackError payment-errors"></div>';

        $address = [
          'address_line1' => $order->billing['street_address'],
          'address_city' => tep_output_string($order->billing['city']),
          'address_zip' => tep_output_string($order->billing['postcode']),
          'address_state' => tep_output_string(tep_get_zone_name($order->billing['country_id'], $order->billing['zone_id'], $order->billing['state'])),
          'address_country' => tep_output_string($order->billing['country']['iso_code_2']),
        ];

        foreach ($address as $k => $v) {
            $content .= '<input type="hidden" id="' . tep_output_string($k) . '" value="' . tep_output_string($v) . '" />';
        }
        $content .= '<input type="hidden" id="email_address" value="' . tep_output_string($order->customer['email_address']) . '" />';
        $content .= '<input type="hidden" id="customer_id" value="' . tep_output_string($_SESSION['customer_id']) . '" />';

        if (MODULE_PAYMENT_STRIPE_SCA_TRANSACTION_METHOD == 'Capture') {
            $capture_method = 'automatic';
        } else {
            $capture_method = 'manual';
        }

        // have to create intent before loading the javascript because it needs the intent id
        if (isset($_SESSION['stripe_payment_intent_id'])) {
            try {
                $this->intent = \Stripe\PaymentIntent::retrieve(['id' => $_SESSION['stripe_payment_intent_id']]);
                $this->event_log($_SESSION['customer_id'], 'page retrieve intent', $_SESSION['stripe_payment_intent_id'], $this->intent);
                $this->intent->amount = $this->format_raw($order->info['total']);
                $this->intent->currency = $currency;
                $this->intent->metadata = $metadata;
                $response = $this->intent->save();
            } catch (exception $err) {
                $this->event_log($_SESSION['customer_id'], 'page create intent', $_SESSION['stripe_payment_intent_id'], $err->getMessage());
                // failed to save existing intent, so create new one
                unset($_SESSION['stripe_payment_intent_id']);
            }
        }
        if (!isset($_SESSION['stripe_payment_intent_id'])) {
            $params = [
              'amount' => $this->format_raw($order->info['total']),
              'currency' => $currency,
              'setup_future_usage' => 'off_session',
              'capture_method' => $capture_method,
              'metadata' => $metadata,
            ];
            $this->intent = \Stripe\PaymentIntent::create($params);
            $this->event_log($_SESSION['customer_id'], 'page create intent', json_encode($params), $this->intent);
            $_SESSION['stripe_payment_intent_id'] = $this->intent->id;
        }
        $content .= '<input type="hidden" id="intent_id" value="' . tep_output_string($_SESSION['stripe_payment_intent_id']) . '" />' .
                '<input type="hidden" id="secret" value="' . tep_output_string($this->intent->client_secret) . '" />';

        if (!$this->templateClassExists()) {
            $content .= $this->getSubmitCardDetailsJavascript();
        }

        $confirmation = ['title' => $content];

        return $confirmation;
    }

    function before_process() {

        $this->after_process();
    }

    function after_process() {

        if (isset($_SESSION['cart_Stripe_SCA_ID'])) {
          $GLOBALS['order']->set_id($this->extract_order_id());
          $GLOBALS['hooks']->register_pipeline('after');

          $GLOBALS['hooks']->register_pipeline('reset');
          unset($_SESSION['stripe_error']);
          unset($_SESSION['stripe_payment_intent_id']);
          unset($_SESSION['cart_Stripe_SCA_ID']);

          tep_redirect(tep_href_link('checkout_success.php'));
        }
    }

    function get_error() {
        global $stripe_error;

        $message = MODULE_PAYMENT_STRIPE_SCA_ERROR_GENERAL;

        if (isset($_SESSION['stripe_error'])) {
            $message = $stripe_error . ' ' . $message;

            unset($_SESSION['stripe_error']);
        }

        if (!empty($_GET['error'])) {
            switch ($_GET['error']) {
                case 'cardstored':
                    $message = MODULE_PAYMENT_STRIPE_SCA_ERROR_CARDSTORED;
                    break;
            }
        }

        $error = [
          'title' => MODULE_PAYMENT_STRIPE_SCA_ERROR_TITLE,
          'error' => $message,
        ];

        return $error;
    }

    function event_log($customer_id, $action, $request, $response) {
        if (MODULE_PAYMENT_STRIPE_SCA_LOG == 'True') {
            tep_db_query("INSERT into stripe_event_log (customer_id, action, request, response, date_added) VALUES ('" . $customer_id . "', '" . $action . "', '" . tep_db_input($request) . "', '" . tep_db_input($response) . "', now())");
        }
    }

    function get_parameters() {
        if (tep_db_num_rows(tep_db_query("SHOW TABLES LIKE 'customers_stripe_tokens'")) != 1) {
            $sql = <<<EOD
CREATE TABLE customers_stripe_tokens (
  id int NOT NULL auto_increment,
  customers_id int NOT NULL,
  stripe_token varchar(255) NOT NULL,
  card_type varchar(32) NOT NULL,
  number_filtered varchar(20) NOT NULL,
  expiry_date char(6) NOT NULL,
  date_added datetime NOT NULL,
  PRIMARY KEY (id),
  KEY idx_cstripet_customers_id (customers_id),
  KEY idx_cstripet_token (stripe_token)
);
EOD;

            tep_db_query($sql);
        }
        if (tep_db_num_rows(tep_db_query("SHOW TABLES LIKE 'stripe_event_log'")) != 1) {
            $sql = <<<EOD
CREATE TABLE stripe_event_log (
  id int NOT NULL auto_increment,
  customer_id int NOT NULL,
  action varchar(255) NOT NULL,
  request varchar(255) NOT NULL,
  response varchar(255) NOT NULL,
  date_added datetime NOT NULL,
  PRIMARY KEY (id)
);
EOD;

            tep_db_query($sql);
        }

        $params = [
            'MODULE_PAYMENT_STRIPE_SCA_STATUS' => [
              'title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_STATUS_TITLE,
              'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_STATUS_DESC,
              'value' => 'True',
              'set_func' => 'tep_cfg_select_option(array(\'True\', \'False\'), ',
            ],
            'MODULE_PAYMENT_STRIPE_SCA_TRANSACTION_SERVER' => [
              'title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_SERVER_TITLE,
              'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_SERVER_DESC,
              'value' => 'Live',
              'set_func' => 'tep_cfg_select_option(array(\'Live\', \'Test\'), ',
            ],
            'MODULE_PAYMENT_STRIPE_SCA_LIVE_PUBLISHABLE_KEY' => [
              'title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_LIVE_PUB_TITLE,
              'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_LIVE_PUB_DESC,
              'value' => '',
            ],
            'MODULE_PAYMENT_STRIPE_SCA_LIVE_SECRET_KEY' => [
                'title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_LIVE_SECRET_TITLE,
                'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_LIVE_SECRET_DESC,
                'value' => '',
            ],
            'MODULE_PAYMENT_STRIPE_SCA_LIVE_WEBHOOK_SECRET' => [
                'title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_LIVE_WEBHOOK_TITLE,
                'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_LIVE_WEBHOOK_DESC,
                'value' => '',
            ],
            'MODULE_PAYMENT_STRIPE_SCA_TEST_PUBLISHABLE_KEY' => [
                'title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_TEST_PUB_TITLE,
                'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_TEST_PUB_DESC,
                'value' => ''],
            'MODULE_PAYMENT_STRIPE_SCA_TEST_SECRET_KEY' => [
                'title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_TEST_SECRET_TITLE,
                'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_TEST_SECRET_DESC,
                'value' => ''],
            'MODULE_PAYMENT_STRIPE_SCA_TEST_WEBHOOK_SECRET' => [
                'title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_TEST_WEBHOOK_TITLE,
                'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_TEST_WEBHOOK_DESC,
                'value' => ''],
            'MODULE_PAYMENT_STRIPE_SCA_TOKENS' => [
                'title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_TOKENS_TITLE,
                'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_TOKENS_DESC,
                'value' => 'False',
                'set_func' => 'tep_cfg_select_option(array(\'True\', \'False\'), '],
            'MODULE_PAYMENT_STRIPE_SCA_LOG' => [
                'title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_LOG_TITLE,
                'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_LOG_DESC,
                'value' => 'False',
                'set_func' => 'tep_cfg_select_option(array(\'True\', \'False\'), '],
            'MODULE_PAYMENT_STRIPE_SCA_TRANSACTION_METHOD' => ['title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_METHOD_TITLE,
                'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_METHOD_DESC,
                'value' => 'Authorize',
                'set_func' => 'tep_cfg_select_option(array(\'Authorize\', \'Capture\'), '],
            'MODULE_PAYMENT_STRIPE_SCA_PREPARE_ORDER_STATUS_ID' => [
                'title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_NEW_ORDER_TITLE,
                'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_NEW_ORDER_DESC,
                'value' => abstract_payment_module::ensure_order_status('MODULE_PAYMENT_STRIPE_SCA_PREPARE_ORDER_STATUS_ID', 'Preparing [Stripe SCA]'),
                'use_func' => 'tep_get_order_status_name',
                'set_func' => 'tep_cfg_pull_down_order_statuses('],
            'MODULE_PAYMENT_STRIPE_SCA_ORDER_STATUS_ID' => [
                'title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_PROCESSED_TITLE,
                'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_PROCESSED_DESC,
                'value' => '0',
                'use_func' => 'tep_get_order_status_name',
                'set_func' => 'tep_cfg_pull_down_order_statuses('],
            'MODULE_PAYMENT_STRIPE_SCA_TRANSACTION_ORDER_STATUS_ID' => [
                'title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_TRANSACTION_TITLE,
                'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_TRANSACTION_DESC,
                'value' => abstract_payment_module::ensure_order_status('MODULE_PAYMENT_STRIPE_SCA_TRANSACTION_ORDER_STATUS_ID', 'Stripe SCA [Transactions]'),
                'set_func' => 'tep_cfg_pull_down_order_statuses(',
                'use_func' => 'tep_get_order_status_name'],
            'MODULE_PAYMENT_STRIPE_SCA_ZONE' => [
                'title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_ZONE_TITLE,
                'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_ZONE_DESC,
                'value' => '0',
                'use_func' => 'tep_get_zone_class_title',
                'set_func' => 'tep_cfg_pull_down_zone_classes('],
            'MODULE_PAYMENT_STRIPE_SCA_VERIFY_SSL' => [
                'title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_SSL_TITLE,
                'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_SSL_DESC,
                'value' => 'True',
                'set_func' => 'tep_cfg_select_option(array(\'True\', \'False\'), '],
            'MODULE_PAYMENT_STRIPE_SCA_PROXY' => [
                'title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_PROXY_TITLE,
                'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_PROXY_DESC],
            'MODULE_PAYMENT_STRIPE_SCA_DEBUG_EMAIL' => [
                'title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_EMAIL_TITLE,
                'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_EMAIL_DESC],
            'MODULE_PAYMENT_STRIPE_SCA_SORT_ORDER' => [
                'title' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_SORT_TITLE,
                'desc' => MODULE_PAYMENT_STRIPE_SCA_ADMIN_SOR_DESC,
                'value' => '0'],
        ];

        return $params;
    }

    function sendTransactionToGateway($url, $parameters = null, $curl_opts = []) {
        $server = parse_url($url);

        if (isset($server['port']) === false) {
            $server['port'] = ($server['scheme'] == 'https') ? 443 : 80;
        }

        if (isset($server['path']) === false) {
            $server['path'] = '/';
        }

        $header = [
          'Stripe-Version: ' . $this->api_version,
          'User-Agent: OSCOM ' . tep_get_version(),
        ];

        if (is_array($parameters) && !empty($parameters)) {
            $post_string = '';

            foreach ($parameters as $key => $value) {
                $post_string .= $key . '=' . urlencode(utf8_encode(trim($value))) . '&';
            }

            $post_string = substr($post_string, 0, -1);

            $parameters = $post_string;
        }

        $curl = curl_init($server['scheme'] . '://' . $server['host'] . $server['path'] . (isset($server['query']) ? '?' . $server['query'] : ''));
        curl_setopt($curl, CURLOPT_PORT, $server['port']);
        curl_setopt($curl, CURLOPT_HEADER, false);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_FORBID_REUSE, true);
        curl_setopt($curl, CURLOPT_FRESH_CONNECT, true);
        curl_setopt($curl, CURLOPT_USERPWD, MODULE_PAYMENT_STRIPE_SCA_TRANSACTION_SERVER == 'Live' ? MODULE_PAYMENT_STRIPE_SCA_LIVE_SECRET_KEY : MODULE_PAYMENT_STRIPE_SCA_TEST_SECRET_KEY . ':');
        curl_setopt($curl, CURLOPT_HTTPHEADER, $header);

        if (!empty($parameters)) {
            curl_setopt($curl, CURLOPT_POST, true);
            curl_setopt($curl, CURLOPT_POSTFIELDS, $parameters);
        }

        if (MODULE_PAYMENT_STRIPE_SCA_VERIFY_SSL == 'True') {
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, true);
            curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 2);

            if (file_exists(DIR_FS_CATALOG . 'ext/modules/payment/stripe/data/ca-certificates.crt')) {
                curl_setopt($curl, CURLOPT_CAINFO, DIR_FS_CATALOG . 'ext/modules/payment/stripe/data/ca-certificates.crt');
            } elseif (file_exists(DIR_FS_CATALOG . 'includes/cacert.pem')) {
                curl_setopt($curl, CURLOPT_CAINFO, DIR_FS_CATALOG . 'includes/cacert.pem');
            }
        } else {
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        }

        if (tep_not_null(MODULE_PAYMENT_STRIPE_SCA_PROXY)) {
            curl_setopt($curl, CURLOPT_HTTPPROXYTUNNEL, true);
            curl_setopt($curl, CURLOPT_PROXY, MODULE_PAYMENT_STRIPE_SCA_PROXY);
        }

        if (!empty($curl_opts)) {
            foreach ($curl_opts as $key => $value) {
                curl_setopt($curl, $key, $value);
            }
        }

        $result = curl_exec($curl);

        curl_close($curl);

        return $result;
    }

    function getTestLinkInfo() {
        $dialog_title = MODULE_PAYMENT_STRIPE_SCA_DIALOG_CONNECTION_TITLE;
        $dialog_button_close = MODULE_PAYMENT_STRIPE_SCA_DIALOG_CONNECTION_BUTTON_CLOSE;
        $dialog_success = MODULE_PAYMENT_STRIPE_SCA_DIALOG_CONNECTION_SUCCESS;
        $dialog_failed = MODULE_PAYMENT_STRIPE_SCA_DIALOG_CONNECTION_FAILED;
        $dialog_error = MODULE_PAYMENT_STRIPE_SCA_DIALOG_CONNECTION_ERROR;
        $dialog_connection_time = MODULE_PAYMENT_STRIPE_SCA_DIALOG_CONNECTION_TIME;

        $test_url = tep_href_link('modules.php', 'set=payment&module=' . $this->code . '&action=install&subaction=conntest');

        $js = <<<EOD
<script>
$(function() {
  $('#tcdprogressbar').progressbar({
    value: false
  });
});

function openTestConnectionDialog() {
  var d = $('<div>').html($('#testConnectionDialog').html()).dialog({
    modal: true,
    title: '{$dialog_title}',
    buttons: {
      '{$dialog_button_close}': function () {
        $(this).dialog('destroy');
      }
    }
  });

  var timeStart = new Date().getTime();

  $.ajax({
    url: '{$test_url}'
  }).done(function(data) {
    if ( data == '1' ) {
      d.find('#testConnectionDialogProgress').html('<p style="font-weight: bold; color: green;">{$dialog_success}</p>');
    } else {
      d.find('#testConnectionDialogProgress').html('<p style="font-weight: bold; color: red;">{$dialog_failed}</p>');
    }
  }).fail(function() {
    d.find('#testConnectionDialogProgress').html('<p style="font-weight: bold; color: red;">{$dialog_error}</p>');
  }).always(function() {
    var timeEnd = new Date().getTime();
    var timeTook = new Date(0, 0, 0, 0, 0, 0, timeEnd-timeStart);

    d.find('#testConnectionDialogProgress').append('<p>{$dialog_connection_time} ' + timeTook.getSeconds() + '.' + timeTook.getMilliseconds() + 's</p>');
  });
}
</script>
EOD;

        $info = '<p><i class="fas fa-lock"></i>&nbsp;<a href="javascript:openTestConnectionDialog();" style="text-decoration: underline; font-weight: bold;">' . MODULE_PAYMENT_STRIPE_SCA_DIALOG_CONNECTION_LINK_TITLE . '</a></p>' .
                '<div id="testConnectionDialog" style="display: none;"><p>Server:<br />https://api.stripe.com/v3/</p><div id="testConnectionDialogProgress"><p>' . MODULE_PAYMENT_STRIPE_SCA_DIALOG_CONNECTION_GENERAL_TEXT . '</p><div id="tcdprogressbar"></div></div></div>' .
                $js;

        return $info;
    }

    function getTestConnectionResult() {
        $stripe_result = json_decode($this->sendTransactionToGateway('https://api.stripe.com/v3/charges/oscommerce_connection_test'), true);

        if (is_array($stripe_result) && !empty($stripe_result) && isset($stripe_result['error'])) {
            return 1;
        }

        return -1;
    }

    function format_raw($number, $currency_code = '', $currency_value = '') {
        global $currencies, $currency;

        if (empty($currency_code) || !$currencies->is_set($currency_code)) {
            $currency_code = $currency;
        }

        if (empty($currency_value) || !is_numeric($currency_value)) {
            $currency_value = $currencies->currencies[$currency_code]['value'];
        }

        return number_format(tep_round($number * $currency_value, $currencies->currencies[$currency_code]['decimal_places']), 2, '', '');
    }

    function templateClassExists() {
        return isset($GLOBALS['oscTemplate']) && ($GLOBALS['oscTemplate'] instanceof oscTemplate);
    }

    function getSubmitCardDetailsJavascript($intent = null) {
        $stripe_publishable_key = MODULE_PAYMENT_STRIPE_SCA_TRANSACTION_SERVER == 'Live' ? MODULE_PAYMENT_STRIPE_SCA_LIVE_PUBLISHABLE_KEY : MODULE_PAYMENT_STRIPE_SCA_TEST_PUBLISHABLE_KEY;
        $intent_url = tep_href_link("ext/modules/payment/stripe_sca/payment_intent.php", '', 'SSL', false, false);
        $js = <<<EOD
<style>
#stripe_table_new_card #card-element {
  background-color: #fff;
  padding: 6px 12px;
  border: 1px solid #ccc;
  border-radius: 4px;
}
</style>
<script src="https://js.stripe.com/v3/"></script>
<script>
$(function() {
    $('[name=checkout_confirmation]').attr('id','payment-form');

    var stripe = Stripe('{$stripe_publishable_key}');
    var elements = stripe.elements();

    // Create an instance of the card Element.
    var card = elements.create('card', {hidePostalCode: true});

    // Add an instance of the card Element into the `card-element` <div>.
    card.mount('#card-element');

    $('#payment-form').submit(function(event) {
        var \$form = $(this);

        // Disable the submit button to prevent repeated clicks
        \$form.find('button').prop('disabled', true);

        var selected = $("input[name='stripe_card']:checked").val();
        var cc_save = $('[name=card-save]').prop('checked');
        if (typeof selected === 'undefined') {
            selected = 0;
        }
       try {
            if ((selected != null && selected != '0') || cc_save) {
                // update intent to use saved card, then process payment if successful
                updatePaymentIntent(cc_save,selected);
            } else {
                // using new card details without save
                processNewCardPayment();
            }
        } catch ( error ) {
            \$form.find('.payment-errors').text(error);
        }

        // Prevent the form from submitting with the default action
        return false;
    });

    if ( $('#stripe_table').length > 0 ) {
        if ( typeof($('#stripe_table').parent().closest('table').attr('width')) == 'undefined' ) {
          $('#stripe_table').parent().closest('table').attr('width', '100%');
        }

        $('#stripe_table .moduleRowExtra').hide();

        $('#stripe_table_new_card').hide();
        $('#card-element').prop('id','new-card-element');
        $('#save-card-element').prop('id','card-element');

        $('form[name="checkout_confirmation"] input[name="stripe_card"]').change(function() {

            if ( $(this).val() == '0' ) {
                stripeShowNewCardFields();
            } else {
                if ($('#stripe_table_new_card').is(':visible')) {
                    $('#card-element').prop('id','new-card-element');
                    $('#save-card-element').prop('id','card-element');
                }
                $('#stripe_table_new_card').hide();

            }
            $('tr[id^="stripe_card_"]').removeClass('moduleRowSelected');
            $('#stripe_card_' + $(this).val()).addClass('moduleRowSelected');
            });

        $('form[name="checkout_confirmation"] input[name="stripe_card"]:first').prop('checked', true).trigger('change');

        $('#stripe_table .moduleRow').hover(function() {
            $(this).addClass('moduleRowOver');
        }, function() {
            $(this).removeClass('moduleRowOver');
        }).click(function(event) {
            var target = $(event.target);

            if ( !target.is('input:radio') ) {
                $(this).find('input:radio').each(function() {
                    if ( $(this).prop('checked') == false ) {
                        $(this).prop('checked', true).trigger('change');
                    }
                });
            }
            });
    } else {
        if ( typeof($('#stripe_table_new_card').parent().closest('table').attr('width')) == 'undefined' ) {
            $('#stripe_table_new_card').parent().closest('table').attr('width', '100%');
        }
    }
    function stripeShowNewCardFields() {

        $('#card-element').attr('id','save-card-element');
        $('#new-card-element').attr('id','card-element');

        $('#stripe_table_new_card').show();
    }
    function updatePaymentIntent(cc_save,token){
        // add card save option to payment intent, so card can be saved in webhook
        // or customer/payment method if using saved card
        $.getJSON( "{$intent_url}",{"id":$('#intent_id').val(),
                                    "token":token,
                                    "customer_id": $('#customer_id').val(),
                                    "cc_save": cc_save},
        function( data ) {
            if (data.status == 'ok') {
                var selected = $("input[name='stripe_card']:checked"). val();

                if (selected == null || selected == '0') {
                    processNewCardPayment();
                } else {
                    processSavedCardPayment(data.payment_method);
                }
            } else {
                var \$form = $('#payment-form');
                \$form.find('button').prop('disabled', false);
                $('#card-errors').text(data.error);
            }
        });
    }
    function processNewCardPayment() {
        stripe.handleCardPayment(
            $('#secret').val(), card, {
              payment_method_data: {
                billing_details: {
                    name: $('#cardholder-name').val(),
                    address: {
                        city: $('#address_city').val(),
                        line1: $('#address_line1').val(),
                        postal_code: $('#address_zip').val(),
                        state: $('#address_state').val(),
                        country: $('#address_country').val()
                    },
                    email: $('#email_address').val()
                }
              }
            }
        ).then(function(result) {
            stripeResponseHandler(result);
        });
    }
    function processSavedCardPayment(payment_method_id) {
        stripe.handleCardPayment(
            $('#secret').val(),
            {
              payment_method: payment_method_id
            }
        ).then(function(result) {
            stripeResponseHandler(result);
        });
    }
    function stripeResponseHandler(result) {
        var \$form = $('#payment-form');
        if (result.error) {
            $('#card-errors').text(result.error.message);
            \$form.find('button').prop('disabled', false);
        } else {
            $('#card-errors').text('Processing');

            // Insert the token into the form so it gets submitted to the server
            \$form.append($('<input type="hidden" name="stripeIntentId" />').val(result.paymentIntent.id));
            // and submit
            \$form.get(0).submit();
        }
    }
});
</script>
EOD;

        return $js;
    }

    function sendDebugEmail($response = []) {
        if (tep_not_null(MODULE_PAYMENT_STRIPE_SCA_DEBUG_EMAIL)) {
            $email_body = '';

            if (!empty($response)) {
                $email_body .= 'RESPONSE:' . "\n\n" . print_r($response, true) . "\n\n";
            }

            if (!empty($_POST)) {
                $email_body .= '$_POST:' . "\n\n" . print_r($_POST, true) . "\n\n";
            }

            if (!empty($_GET)) {
                $email_body .= '$_GET:' . "\n\n" . print_r($_GET, true) . "\n\n";
            }

            if (!empty($email_body)) {
                tep_mail('', MODULE_PAYMENT_STRIPE_SCA_DEBUG_EMAIL, 'Stripe Debug E-Mail', trim($email_body), STORE_OWNER, STORE_OWNER_EMAIL_ADDRESS);
            }
        }
    }

    function deleteCard($card, $customer, $token_id) {
        $secret_key = MODULE_PAYMENT_STRIPE_SCA_TRANSACTION_SERVER == 'Live' ? MODULE_PAYMENT_STRIPE_SCA_LIVE_SECRET_KEY : MODULE_PAYMENT_STRIPE_SCA_TEST_SECRET_KEY;
        \Stripe\Stripe::setApiKey($secret_key);
        \Stripe\Stripe::setApiVersion($this->api_version);
        $error = '';
        $payment_method = \Stripe\PaymentMethod::retrieve($card);
        try {
            $result = $payment_method->detach();
        } catch (exception $err) {
            // just log error, and continue to delete card from table
            $error = $err->getMessage();
        }

        $this->event_log($_SESSION['customer_id'], "deleteCard", $payment_method, $error);

        if (!isset($result->object) || ($result->object !== 'payment_method')) {
            $this->sendDebugEmail($result . PHP_EOL . $error);
        }

        tep_db_query("delete from customers_stripe_tokens where id = '" . (int) $token_id . "' and customers_id = '" . (int) $_SESSION['customer_id'] . "' and stripe_token = '" . tep_db_input(tep_db_prepare_input($customer . ':|:' . $card)) . "'");

        return (tep_db_affected_rows() === 1);
    }

}
